package com.persagy.apm.energy.report.common.service;

import com.persagy.apm.energy.report.common.dto.QueryPlatformParamDTO;
import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.StatisticItems;
import com.persagy.apm.energy.report.monthly.detail.business.model.Platform;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectAlarmDealInfo;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectKpiItem;
import io.swagger.annotations.Api;

import java.util.Date;
import java.util.List;

@Api(value = "公共service接口")
public interface CommonService {

    /**
     * @param pjIdList 项目id列表
     * @return
     */
    List<String> getPjNameByPjId(List<String> pjIdList);

    /**
     * 根据配置条目信息获取能耗管理平台运维数据
     *
     * @param statisticItems        条目信息
     * @param queryPlatformParamDTO 查询条件对象
     * @return 能耗管理平台运维数据
     */
    Platform getPlatform(StatisticItems statisticItems, QueryPlatformParamDTO queryPlatformParamDTO);

    /**
     * 获取酒店项目报警处理信息
     *
     * @param projectId 项目id
     * @param reportDate 报告月
     * @return 酒店项目报警处理信息
     */
    HotelProjectAlarmDealInfo getHotelProjectAlarmDealInfo(String projectId, Date reportDate);

    /**
     * 获取酒店项目环境指标条目信息
     *
     * @param projectId 项目id
     * @param reportDate 报告月
     * @param projectIdList 当前报告类型下的所有项目id列表
     * @param function  function信息
     * @return 酒店项目环境指标条目信息
     */
    HotelProjectKpiItem getHotelProjectKpiItem(String projectId, Date reportDate, List<String> projectIdList, Function function);

}
