package com.persagy.apm.energy.report.common.service.impl;

import com.persagy.apm.common.model.dto.PoemsPuser;
import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.common.DataConstants;
import com.persagy.apm.energy.report.common.dto.QueryPlatformParamDTO;
import com.persagy.apm.energy.report.common.service.CommonService;
import com.persagy.apm.energy.report.common.service.PlatformService;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.common.utils.SpringContextHolder;
import com.persagy.apm.energy.report.common.vo.PlatformAreaInfoVO;
import com.persagy.apm.energy.report.environment.model.dto.BatchQueryEnvListByParamDTO;
import com.persagy.apm.energy.report.environment.service.IEnvironmentWebService;
import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.StatisticItems;
import com.persagy.apm.energy.report.monthly.config.type.constant.enums.BelongTypeEnum;
import com.persagy.apm.energy.report.monthly.detail.business.model.Platform;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectAlarmDealInfo;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectKpiItem;
import com.persagy.apm.energy.report.saasweb.model.dto.QueryProjectDTO;
import com.persagy.apm.energy.report.saasweb.model.vo.PoemsProjectVO;
import com.persagy.apm.energy.report.saasweb.service.impl.SaasWebClientWrapper;
import com.persagy.apm.energy.report.workorder.service.IWorkOrderWebService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

@Api(value = "公共service实现类")
@Service
@Slf4j
public class CommonServiceImpl implements CommonService {

    @Autowired
    SaasWebClientWrapper saasWebClientWrapper;

    @Autowired
    SpringContextHolder holder;

    @Autowired
    IWorkOrderWebService iWorkOrderWebService;

    @Autowired
    IEnvironmentWebService iEnvironmentWebService;

    @Override
    public List<String> getPjNameByPjId(List<String> pjIdList) {
        if (CollectionUtils.isEmpty(pjIdList)) {
            return new ArrayList<>();
        }
        QueryProjectDTO queryProjectDTO = new QueryProjectDTO();
        PoemsPuser poemsPuser = new PoemsPuser();
        poemsPuser.setUserId(DataConstants.USER_ID);
        poemsPuser.setLoginDevice(DataConstants.LOGIN_DEVICE);
        poemsPuser.setPd(DataConstants.PD);
        queryProjectDTO.setPuser(poemsPuser);
        PoemsFeignResponse<PoemsProjectVO> projectsResponse = saasWebClientWrapper.queryProjects(queryProjectDTO);
        List<PoemsProjectVO> projectVOList = projectsResponse.getContent();
        if (!CollectionUtils.isEmpty(projectVOList)) {
            List<String> pjNameList = new ArrayList<>();
            for (PoemsProjectVO poemsProjectVO : projectVOList) {
                if (!pjIdList.contains(poemsProjectVO.getProjectId())) {
                    continue;
                }
                pjNameList.add(poemsProjectVO.getProjectLocalName());
            }
            return pjNameList;
        }
        return new ArrayList<>();
    }

    @Override
    public Platform getPlatform(StatisticItems statisticItems, QueryPlatformParamDTO queryPlatformParamDTO) {
        String serviceName = statisticItems.getItemCode() + statisticItems.getItemSource() + "WebService";
        PlatformService platformService = holder.getPlatformService(serviceName);
        Platform platform = new Platform();
        platform.setItemCode(statisticItems.getItemCode());
        platform.setModelName(statisticItems.getItemName());
        if (queryPlatformParamDTO.getBelongType().equals(BelongTypeEnum.PROJECT.getType())) {
            platform.setMonthlySummaries(platformService.getPjPlatformRate(queryPlatformParamDTO));
        } else if (queryPlatformParamDTO.getBelongType().equals(BelongTypeEnum.AREA.getType())) {
            // 合格标准
            String standardValueStr = statisticItems.getStandardValue();
            Double standardValue = DataUtils.parseDouble(standardValueStr);
            platform.setStandardValue(standardValue);
            platform.setQualifyFormula(statisticItems.getQualifyFormula());
            platform.setStandardFlag(statisticItems.getStandardFlag());
            PlatformAreaInfoVO platformAreaInfoVO = platformService.getAreaPlatformRate(statisticItems, queryPlatformParamDTO);
            if (null != platformAreaInfoVO) {
                platform.setSatisfiedCount(platformAreaInfoVO.getSatisfiedCount());
                platform.setUnsatisfiedCount(platformAreaInfoVO.getUnsatisfiedCount());
                platform.setUnsatisfiedProjects(platformAreaInfoVO.getUnsatisfiedProjects());
            }
        }
        return platform;
    }

    @Override
    public HotelProjectAlarmDealInfo getHotelProjectAlarmDealInfo(String projectId, Date reportDate) {
        Integer workCount = iWorkOrderWebService.getWorkOrderNumByParam(projectId, reportDate, "ask_end_time");
        Integer dealCount = iWorkOrderWebService.getWorkOrderNumByParam(projectId, reportDate, "create_time");
        Double completeRate = null;
        if (null != workCount && workCount != 0 && null != dealCount) {
            completeRate = DataUtils.parseDouble(dealCount) / workCount * 100;
        }
        HotelProjectAlarmDealInfo hotelProjectAlarmDealInfo = new HotelProjectAlarmDealInfo();
        hotelProjectAlarmDealInfo.setHandledCount(dealCount);
        hotelProjectAlarmDealInfo.setHandledPercent(completeRate);
        hotelProjectAlarmDealInfo.setTotalCount(workCount);
        return hotelProjectAlarmDealInfo;
    }

    @Override
    public HotelProjectKpiItem getHotelProjectKpiItem(String projectId, Date reportDate, List<String> projectIdList, Function function) {
        if (CollectionUtils.isEmpty(projectIdList) || StringUtils.isEmpty(projectId) || null == reportDate || null == function || StringUtils.isEmpty(function.getItemId())) {
            return new HotelProjectKpiItem();
        }
        try {
            String startTime = DateUtils.date2Str(reportDate, DateUtils.SDF_SECOND);
            String endTime = DateUtils.date2Str(DateUtils.getMonthOff(reportDate, 1), DateUtils.SDF_SECOND);
            String itemId = function.getItemId();
            String[] itemIdList = itemId.split("_");
            String envTypdId = itemIdList[0];
            String envParentTypeId = itemIdList[1];
            BatchQueryEnvListByParamDTO batchQueryEnvListByParamDTO = new BatchQueryEnvListByParamDTO();
            batchQueryEnvListByParamDTO.setProjectIdList(projectIdList);
            batchQueryEnvListByParamDTO.setEnvTypeId(envTypdId);
            batchQueryEnvListByParamDTO.setEnvParentTypeId(envParentTypeId);
            batchQueryEnvListByParamDTO.setStartTime(startTime);
            batchQueryEnvListByParamDTO.setEndTime(endTime);
            List<HotelProjectKpiItem> hotelProjectKpiItemList = iEnvironmentWebService.getPjEnvRateInfoMap(batchQueryEnvListByParamDTO);
            Map<String, Integer> sortMap = new HashMap<>();
            sortMap.put("controlRates", 1);
            DataUtils.sort(hotelProjectKpiItemList, sortMap);
            for (int i = 0; i < hotelProjectKpiItemList.size(); i++) {
                HotelProjectKpiItem hotelProjectKpiItem = hotelProjectKpiItemList.get(i);
                if (!hotelProjectKpiItem.getProjectId().equals(projectId)) {
                    continue;
                }
                hotelProjectKpiItem.setRankWithinClimateZone(i + 1);
                return hotelProjectKpiItem;
            }
        } catch (Exception e) {
            log.error("获取HotelProjectKpiItem的" + function.getItemId() + "数据失败：", e);
        }
        return new HotelProjectKpiItem();
    }

}
